package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.GuiGlobals;
import dev.pfaff.altargui.internal.render.GLWrapper;
import dev.pfaff.altargui.render.GLFlags;
import dev.pfaff.glass.opengl.enums.GLGetPName;

import static dev.pfaff.altargui.GuiGlobals.GL_WRAPPER;

public interface Uniform {
	UniformHandle handle();

	record OfVec2f(UniformHandle handle) implements Uniform {
		/**
		 * Sets the value of the uniform. If {@link GLWrapper#canUseGlProgramUniformFunctions()} is {@code false}, the
		 * caller must ensure that the {@link #handle() shader}'s {@link UniformHandle#shaderProg() shader} is bound
		 * before calling this method.
		 */
		public void set(float x, float y) {
			if (GL_WRAPPER.canUseGlProgramUniformFunctions()) {
				GuiGlobals.GL.glProgramUniform2f(handle.shaderProg().handle(), handle.location(), x, y);
			} else {
				if (GLFlags.EXPENSIVE_CHECKS) {
					int program = GuiGlobals.GL.glGetInteger(GLGetPName.CURRENT_PROGRAM);
					if (program != handle.shaderProg().handle()) {
						throw new IllegalStateException(STR."Expected the active program to be \{handle.shaderProg()}, found \{program}");
					}
				}
				GuiGlobals.GL.glUniform2f(handle.location(), x, y);
			}
		}
	}
}
